# Copyright 2013, Red Hat, Inc
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License along
# with this program; if not, write to the Free Software Foundation, Inc.,
# 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
#
# Authors:
#   Josef Skladanka <jskladan@redhat.com>

from flask import Blueprint
from flask import request, url_for, render_template, flash, redirect, abort
from fake_fedorainfra import app, db

import fedora.client

main = Blueprint('main', __name__)


from datetime import datetime
import json
from sqlalchemy import desc
from fake_fedorainfra.models.bodhi import BodhiComment
from fake_fedorainfra.lib import util

NUM_PAGE = app.config['NUM_PAGE']

default_response = util.make_default_update()

def get_bodhi_connection():
    return fedora.client.bodhi.BodhiClient()


@app.route('/bodhi/comment', methods=['POST'])
def bodhi_comment():
    current_time = str(datetime.utcnow().strftime('%Y-%m-%d %H:%M:%S'))
    db.session.add(BodhiComment(current_time, str( request.form['title']),
                str(request.form['text']), str(request.form['user_name']),
                int(request.form['karma']), request.form['email'] in ['true', 'True']))
    db.session.commit()
    return json.dumps(default_response)

@app.route('/bodhi/list', methods=['POST','GET'])
def bodhi_list():
    # we need username, release, package, request, status, type_, bugs, mine
    user= release= package= bodhirequest= status= update_type= bugs= mine = None
    limit = 10

    if 'username' in request.values.keys():
        user = str(request.values['username'])
    if 'release' in request.values.keys():
        release = str(request.values['release'])
    if 'package' in request.values.keys():
        package = str(request.values['package'])
    if 'request' in request.values.keys():
        bodhirequest = str(request.values['request'])
    if 'status' in request.values.keys():
        status = str(request.values['status'])
    if 'type_' in request.values.keys():
        update_type = str(request.values['type_'])
    if 'bugs' in request.values.keys():
        bugs = request.values['bugs']
    if 'mine' in request.values.keys():
        mine = str(request.values['mine'])
    if 'tg_paginate_limit' in request.values.keys():
        limit = int(request.values['tg_paginate_limit'])

    try:
        bodhi = get_bodhi_connection()
        result = bodhi.query(package=package, limit=limit, username=user,
                                release=release, request=bodhirequest, status=status,
                                type_=update_type, bugs=bugs, mine=mine)
    except fedora.client.AuthError, e:
        app.logger.exception('bodhi AuthError')
        abort(401)
    except fedora.client.ServerError, e:
        app.logger.exception('bodhi ServerError')
        abort(int(e.code))

    for update in result['updates']:
        raw_comments = search_comments(update['title'])
        comments = [dict(timestamp=row['date'], update=row['update'] ,text=row['text'], author=row['user'],
                    karma=row['karma'], anonymous=False, group=None) for row in raw_comments]
        update['comments'] = comments
    return json.dumps(result)

def search_comments(update):
    c = db.session.query(BodhiComment).filter(BodhiComment.title.like('%%%s%%' % update)).order_by(BodhiComment.id)
    comments = [dict(date=str(row.date), update=row.title, text=row.text, user=row.username,
                karma=row.karma, send_email=row.send_email, id=row.id) for row in c]
    return comments

def get_comments(start=0, num_comments=NUM_PAGE):
    c = db.session.query(BodhiComment).order_by(desc(BodhiComment.id)).slice(start, start+num_comments)
    comments = [dict(date=str(row.date), update=row.title, text=row.text, user=row.username,
                karma=row.karma, send_email=row.send_email, id=row.id ) for row in c]
    return comments

@app.route('/boji/comments', methods=['GET'])
def default_boji_comments():
    return redirect(url_for('boji_comments', start_comment=0))

@app.route('/boji/comments/<int:start_comment>', methods=['GET'])
def boji_comments(start_comment):
    comments = get_comments(start=start_comment)
    for c in comments:
        c['url'] = url_for('get_boji_comment', comment_id=c['id'])
    next_start = (start_comment + NUM_PAGE)
    prev_start = (start_comment - NUM_PAGE)
    if prev_start < 0:
        prev_start = 0
    return render_template('view_comments.html', bodhi_comments=comments, next_start= next_start, prev_start= prev_start)

@app.route('/boji/comments/search', methods=['GET', 'POST'])
def boji_search_comments():
    if request.method == 'GET':
        return render_template('search_comments.html')
    elif request.method == 'POST':
        if 'title' in request.form.keys():
            comments = search_comments(request.form['title'])
        return render_template('view_comments.html', bodhi_comments=comments, next_start= 0, prev_start=0)

@app.route('/boji/comment/<int:comment_id>', methods = ['GET', 'POST', 'DELETE'])
def get_boji_comment(comment_id):
    c = db.session.query(BodhiComment).filter(BodhiComment.id == comment_id).first()

    if request.method == 'GET':
        comment = dict(date=str(c.date), update=c.title, text=c.text, user=c.username, karma=c.karma, send_email=c.send_email, id=c.id)
        return render_template('comment_detail.html', comment = comment)

    # stupid browsers not supporting HTTP delete calls ...
    elif request.method == 'POST':
        if request.form['request'] == 'DELETE':
            db.session.delete(c)
            db.session.commit()
            flash('Comment %d was deleted' % comment_id)
            return redirect(url_for('default_boji_comments'))

    elif request.method == 'DELETE':
        db.session.delete(c)
        db.session.commit()
        return (url_for('default_boji_comments'))

@app.route('/boji/', methods = ['GET'])
def boji_main():
    return render_template('boji_main.html')

@app.route('/util/cleardb', methods=['POST'])
def clear_db():
    db.session.execute('delete from comments')
    db.session.commit()
    flash('Database was cleared')
    return redirect(url_for('default_boji_comments'))

@app.route('/', methods = ['GET'])
def boji_landing():
    return redirect(url_for('boji_main'))

